<?php
declare(strict_types=1);

namespace FiloBlu\Csp\Model;

use Magento\Framework\App\Area;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\State;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;
use FiloBlu\Csp\Api\Data\ConfigurationInterface;
use FiloBlu\Csp\Api\ConfigManagerInterface;
use FiloBlu\Csp\Model\Data\Configuration;

/**
 * @inheritDoc
 */
class ConfigManager implements ConfigManagerInterface
{
    /**
     * @var ScopeConfigInterface
     */
    private $config;

    /**
     * @var Store
     */
    private $storeModel;

    /**
     * @var State
     */
    private $state;

    /**
     * @param ScopeConfigInterface $config
     * @param Store $store
     * @param State $state
     */
    public function __construct(ScopeConfigInterface $config, Store $store, State $state)
    {
        $this->config = $config;
        $this->storeModel = $store;
        $this->state = $state;
    }

    protected function getConfigArea() {
        $area = $this->state->getAreaCode();
        if ($area === Area::AREA_ADMINHTML) {
            $configArea = 'backend';
        } elseif ($area === Area::AREA_FRONTEND) {
            $configArea = 'frontend';
        } else {
            throw new \RuntimeException('CSP can only be configured for storefront or admin area');
        }
        return $configArea;
    }

    public function getPolicyValue($policyConfigPath)
    {
        $configArea = $this->getConfigArea();
        return $this->config->getValue(
            'filoblu_csp/' . $configArea .'/' . $policyConfigPath,
            ScopeInterface::SCOPE_STORE,
            $this->storeModel->getStore()
        );
    }


    public function getConfiguration() :ConfigurationInterface
    {
        $configArea = $this->getConfigArea();
        $isCspEnabled = $this->config->isSetFlag(
            'filoblu_csp/' . $configArea .'/enabled',
            ScopeInterface::SCOPE_STORE,
            $this->storeModel->getStore()
        );

        $reportOnly = $this->config->isSetFlag(
            'filoblu_csp/' . $configArea .'/report_only',
            ScopeInterface::SCOPE_STORE,
            $this->storeModel->getStore()
        );
        $reportUri = $this->config->getValue(
            'filoblu_csp/' . $configArea .'/report_uri',
            ScopeInterface::SCOPE_STORE,
            $this->storeModel->getStore()
        );

        return new Configuration($isCspEnabled, $reportOnly, !empty($reportUri) ? $reportUri : null);
    }
}
